
import math as mt
import numpy as np
from scipy.stats import norm

from finderivatives.derivative import EuropeanOption


#%% Call
class Call(EuropeanOption):
    
    def __init__(self, strike, maturity, position, premium=0):
        super().__init__(strike, maturity, position, premium)
    
    
    def payoff(self):
        self._payoff = np.maximum(0, self._spot-self._strike) * self._position
        return self._payoff
    
    
    def profit(self):
        self._profit = self.payoff() - self._premium * self._position
        return self._profit
    
    
    def pricing_bs(self):
        # Probabilities
        self._d1 = (np.log(self._spot/self._strike) + 
                    (self._r - (self._vol**2)/2) / self._dt) \
            / (self._vol * mt.sqrt(self._dt))
        self._d2 = self._d1 - self._vol * mt.sqrt(self._dt)
        self._n_d1 = norm.cdf(self._d1)
        self._n_d2 = norm.cdf(self._d2)
        # Pricing
        self._pricing = (
            self._spot * self._n_d1\
            - self._strike * np.exp(-self._r*(self._dt)) * self._n_d2
            ) * self._position
        return self._pricing
    
    
    # def payoff_plot



from finderivatives import Call
from finderivatives import Put
from finderivatives import Portfolio


#%% Covered Call


#%% Reverse Covered Call



#%% Protective Put



#%% Revers Protective Put



#%% Bull Spread

class BullSpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%% Bear Spread

class BearSpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%% Butterfly

class ButterflySpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%% Straddle

class StraddleSpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%% Strip

class StripSpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%% Strap

class StrapSpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%% Strangle

class StrangleSpread(Portfolio):
    
    def __init__(self, *derivatives):
        super().__init__(*derivatives)


#%%



#%%








#%%
class Portfolio():
    
    def __init__(self, *derivatives):
        self._derivatives = derivatives
    
    
    def __str__(self):
        return 'xxxxx'
    
    
    def __add__(self, other):
        addition = Portfolio(*self.get_derivatives(), *other.get_derivatives())
        return addition
    
    
    def get_derivatives(self):

        return self._derivatives


    def payoff(self):
        self._payoff = 0
        for derivative in self._derivatives:
            self._payoff += derivative.payoff()
        
        return self._payoff
    
    
    def profit(self):
        self._profit = 0
        for derivative in self._derivatives:
            self._profit += derivative.profit()
        
        return self._profit
    
    
    def pricing_bs(self):
        self._pricing = 0
        for derivative in self._derivatives:
            self._pricing += derivative.pricing_bs()
        
        return self._pricing



#%% Ejecucion directa
if __name__ == '__main__':
    print(' Ejecucion directa ... \n')
    
    from finderivatives import Call, Put
    import numpy as np
    import matplotlib.pyplot as plt
    
    call1 = Call(strike=100, maturity=2, position=1)
    call2 = Call(strike=110, maturity=2, position=1)
    put1 = Put(strike=100, maturity=2, position=1)
    put2 = Put(strike=105, maturity=2, position=-1)
    
    spots = np.arange(80, 120, 1)
    # spots = 102
    
    call1.set_market_inputs(spot=spots, vol=0.06, r=0.01)
    call2.set_market_inputs(spot=spots, vol=0.06, r=0.01)
    put1.set_market_inputs(spot=spots, vol=0.06, r=0.01)
    put2.set_market_inputs(spot=spots, vol=0.06, r=0.01)
    
    port = Portfolio(call1, put1, call2)
    
    payoff_call1 = call1.payoff()
    payoff_call2 = call2.payoff()
    payoff_put1 = put1.payoff()
    
    payoff_port = port.payoff()
    profit_port = port.profit()
    pricing_bs_port = port.pricing_bs()
    
    
    
    
    # print(payoff_call1[39])
    # print(payoff_put1[39])
    # print(payoff_port[39])
    
    
    # fig, ax = plt.subplots()
    # ax.plot(spots, payoff_call1, '--')
    # ax.plot(spots, payoff_call2, '--')
    # ax.plot(spots, payoff_put1, '--')
    # ax.plot(spots, payoff_port, 'o')
    # plt.show()
    
    # derivatives = port.get_derivatives()
    # print(derivatives)
    
    
    #### Portfolio 1
    port1 = Portfolio(call1, put1)
    ### Portfolio 2
    port2 = Portfolio(call2, put2)
    #### Portfolio 3
    port3 = port1 + port2
    
    port3.payoff()


    fig, ax = plt.subplots()
    ax.plot(spots, call1.payoff(), '--')
    ax.plot(spots, call2.payoff(), '--')
    ax.plot(spots, put1.payoff(), '--')
    ax.plot(spots, put2.payoff(), '--')
    ax.plot(spots, payoff_port, 'o')
    plt.show()
    

iter(port1.get_derivatives())



    


"""

"""

import numpy as np
from finderivatives.errors import PositionError, SpotError

    
def validate_position(position):
    try:
        # Validate postion as int or float
        if (isinstance(position, int) | isinstance(position, float)) &\
           (position in [1, -1, 1.0, -1.0]):
            return int(position)
        
        # Validate postion as str
        elif (isinstance(position, str)) & (position in ['1', '-1']):
            return int(position)
        
        # Raise error
        else:
            raise PositionError()
    
    # Catch error
    except PositionError as error:
        error.set_value_error(position)
        raise error



def validate_strike(strike):
    try:
        if (isinstance(strike, int)) | (isinstance(strike, float)):
            return strike
        else:
            raise TypeError
            
    except TypeError:
        message = 'The argument "strike" must be of type int or float'
        raise TypeError(message)


def validate_premium(premium):
    try:
        if (isinstance(premium, int)) | (isinstance(premium, float)):
            return premium
        else:
            raise TypeError
            
    except TypeError:
        message = 'The argument "premium" must be of type int or float'
        raise TypeError(message)



def validate_maturity(maturity):
    try:
        # Validate type
        if not (isinstance(maturity, int)) | (isinstance(maturity, float)):
            raise TypeError
        
        # Validate value
        elif maturity < 0:
            raise ValueError
            
        else:
            return maturity
    
    except TypeError:
        message = 'The argument "maturity" must be of type int or float'
        raise TypeError(message)
        
    except ValueError:
        message = 'The argument "maturity" must be a positive value'
        raise ValueError(message)



def validate_spot(spot):
    try:
        # Validate unique numbers
        if isinstance(spot, int) | isinstance(spot, float):
            return np.array(spot)
        
        # Validate lists and tuples
        elif isinstance(spot, list) | isinstance(spot, tuple):
            for i in spot:
                if isinstance(i, int) | isinstance(i, float):
                    pass
                else:
                    raise SpotError()
            return np.array(spot)
        
        # Validate array
        elif isinstance(spot, np.ndarray):
            return spot
        
        # Raise error
        else:
            raise SpotError()
    
    # Catch error
    except SpotError as error:
        error.set_value_error(spot)
        raise error



if __name__ == '__main__':
    print(' Ejecucion directa ... \n')
    # a = validate_position(0)
    # b = validate_spot([1, 2, 3, 4, '1'])
    # print('Ejecucion completa')










import pandas as pd
import math as mt
import numpy as np
import random as rd
from scipy.stats import norm


#%%
class EuropeanOption():
    
    def __init__(self, strike, maturity, position):
        self._strike = strike
        self._maturity = maturity
        self._position = position
        # self._flows = {}
        
    def get_strike(self):
        return self._strike
    
    def get_maturity(self):
        return self._maturity
    
    def get_position(self):
            return self._position


#%% Call
class Call(EuropeanOption):
    
    def __init__(self, strike, maturity, position):
        super().__init__(strike, maturity, position)
    
    
    def payoff(self, spot):
        self._spot = spot
        self._payoff = max([0, self._spot - self._strike]) * self._position
        return self._payoff
    
    
    def profit(self, spot, premium):
        self._spot = spot
        self._profit = self.payoff(self._spot) - premium * self._position  
        return self._profit
        
    
    def pricing_bs(self, spot, vol, r, dt):
        self._spot = spot
        self._vol = vol
        self._r = r
        self._dt = dt
        # Probabilities
        self._d1 = (mt.log(spot/self._strike) + (r - (vol**2)/2) / dt) \
            / (vol * mt.sqrt(dt))
        self._d2 = self._d1 - vol * mt.sqrt(dt)
        self._n_d1 = norm.cdf(self._d1)
        self._n_d2 = norm.cdf(self._d2)
        # Pricing
        self._pricing = (
            self._spot * self._n_d1\
            - self._strike * np.exp(-self._r*(self._dt)) * self._n_d2
            ) * self._position
        return self._pricing
    


#%%  Put
class Put(EuropeanOption):
    
    def __init__(self, strike, maturity, position):
        super().__init__(strike, maturity, position)
        
    
    def payoff(self, spot):
        self._spot = spot
        self._payoff = max([0, self._strike - self._spot]) * self._position
        return self._payoff
    
    
    def profit(self, spot, premium):
        self._spot = spot
        self._profit = self.payoff(self._spot) - premium * self._position  
        return self._profit
        
    
    def pricing_bs(self, spot, vol, r, dt):
        self._spot = spot
        self._vol = vol
        self._r = r
        self._dt = dt
        # Probabilities
        self._d1 = (mt.log(spot/self._strike) + (r - (vol**2)/2) / dt) \
            / (vol * mt.sqrt(dt))
        self._d2 = self._d1 - vol * mt.sqrt(dt)
        self._n_d1 = norm.cdf(-self._d1)
        self._n_d2 = norm.cdf(-self._d2)
        # Pricing
        self._pricing = (
            self._strike * np.exp(-self._r*(self._dt)) * self._n_d2
            - self._spot * self._n_d1
            ) * self._position
        return self._pricing

    
# # browniano
# mu = 0.05
# sigma = 0.02
# s_0 = 100
# dt = 1/12
# s_1 = s_0 * mt.exp((mu - (sigma**2)*0.5)*dt \
#                    + sigma * mt.sqrt(dt) * rd.normalvariate(0,1))

# print(s_1)





if __name__ == '__main__':
    print(' Ejecucion directa ... \n')
    
    call = Call(strike=100, maturity=2, position=1)
    print('strike:', call.get_strike())
    
    print('payoff:', )

    print('profit:', call.profit(spot=102, premium=4))

    print('pricing:', call.pricing_bs(spot=102, vol=0.04, r=0.01, dt=2))
    
    # s = np.array([96, 98, 100, 102, 104])
    # print('pricing:', call.payoff(spot=s))



strike = 100
position = 1
spot = np.array([96, 98, 100, 102, 104])
zero = 0
payoff = np.maximum([spot - strike], zero) * position
payoff



spots = [96, 98, 100, 102, 104]
spots = list(range(80, 120, 1))

call = Call(strike=100, maturity=2, position=1)
put = Put(strike=100, maturity=2, position=-1)

payoffs_call = [call.payoff(spot=s) for s in spots]
pricings_call = [call.pricing_bs(spot=s, vol=0.02, r=0.01, dt=2) for s in spots]

payoffs_put = [put.payoff(spot=s) for s in spots]
pricings_put = [put.pricing_bs(spot=s, vol=0.02, r=0.01, dt=2) for s in spots]


import matplotlib.pyplot as plt
fig, ax = plt.subplots()
ax.plot(spots, pricings_call)
ax.plot(spots, payoffs_call)
plt.show()


fig, ax = plt.subplots()
ax.plot(spots, pricings_put)
ax.plot(spots, payoffs_put)
plt.show()








import Parche
Parche.main()

import finderivatives as der
import numpy as np
import matplotlib.pyplot as plt

call1 = der.Call(strike=100, maturity=2, position=1, premium=2)
call2 = der.Call(strike=101, maturity=2, position=1, premium=2)
put1 = der.Put(strike=100, maturity=2, position=1, premium=2)

spots = np.arange(80, 120, 1)
call1.set_market_inputs(spot=spots, vol=0.06, r=0.01)
call2.set_market_inputs(spot=spots, vol=0.06, r=0.01)
put1.set_market_inputs(spot=spots, vol=0.06, r=0.01)


#### Portfolio 1
port1 = call1 + call2

#### Plot
# fig, ax = plt.subplots()
# ax.plot(spots, call1.payoff())
# ax.plot(spots, call2.payoff())
# ax.plot(spots, port1['payoff'])
# plt.show()


#### Portfolio 2
port2 = call1 + put1

#### Plot
fig, ax = plt.subplots()
ax.plot(spots, call1.payoff())
ax.plot(spots, put1.payoff())
ax.plot(spots, port2['pricing_bs'])
plt.show()





"""

"""

class PositionError(Exception):
    def __init__(self):
        self.value = None
        self.value_type = None
        self.message = 'The argument "position" must be of type int and take '\
            'the value of 1 or -1. For long position use 1 and for short '\
            'position use -1. '\
            '\nValue received: {value}, type: {value_type}.'
    
    
    def __str__(self):
        return self.message.format(value = self.value,
                                   value_type = self.value_type)
    
    
    def set_value_error(self, value):
        self.value = value
        self.value_type = type(value)



class StrikeError(Exception):
    def __init__(self):
        self.message = 'The argument "position" must be of type int and take '\
            'the value of 1 or -1. For long position use 1 and for short '\
            'position use -1.'
            
    
            

class SpotError(Exception):
    def __init__(self):
        self.value = None
        self.value_type = None
        self.message = 'The argument "spot" must be of type int, list, tuple '\
            'or darray. '\
            '\nValue received: {value}, type: {value_type}.'
    
    
    def __str__(self):
        return self.message.format(value = self.value,
                                   value_type = self.value_type)
    
    
    def set_value_error(self, value):
        self.value = value
        self.value_type = type(value)


if __name__ == '__main__':
    print(' Ejecucion directa ... \n')
    
    raise PositionError()
    print(StrikeError())
    print(SpotError())



"""

pendiente:
    incluir validaciones
    ajustar put
    

"""

from finderivatives import validations as val

#%%
class EuropeanOption():
    
    def __init__(self, strike, maturity, position, premium=0):
        self._strike = val.validate_strike(strike)
        self._maturity = val.validate_maturity(maturity)
        self._dt = val.validate_maturity(maturity)
        self._position = val.validate_position(position)
        self._premium = val.validate_premium(premium)
        # self._flows = {}
    
    
    def __str__(self):
        return 'xxxxx'
    
    
    def __add__(self, other):
        payoff = self.payoff() + other.payoff()
        profit = self.profit() + other.profit()
        pricing_bs = self.pricing_bs() + other.pricing_bs()
        addition = {
            'payoff': payoff,
            'profit': profit,
            'pricing_bs': pricing_bs
            }
        return addition
    
    
    def get_strike(self):
        return self._strike
    
    
    def get_maturity(self):
        return self._maturity
    
    
    def get_position(self):
        return self._position
    
    
    def set_spot(self, spot):
        self._spot = val.validate_spot(spot)
    
    
    def set_volatility(self, vol):
        self._vol = vol
    
    
    def set_r(self, r):
        self._r = r
    
    
    def set_market_inputs(self, spot, vol, r):
        self.set_spot(spot)
        self.set_volatility(vol)
        self.set_r(r)
    


#%% Ejecucion directa
if __name__ == '__main__':
    print(' Ejecucion directa ... \n')
    
    # strike = 105
    # maturity = 2
    # position = 1
    # spot = 102
    # spots = list(range(90, 110, 1))
    # # spots = np.arange(strike*0.8, strike*1.2, strike*0.01)
    # premium = 3
    # vol = 0.05
    # r = 0.01
    # dt = 2
    
    
    # # call = Call(strike=strike, maturity=maturity, position=position,
    #             # premium=premium)
    # call = Put(strike=strike, maturity=maturity, position=position,
    #           premium=premium)
    
    # call.set_spot(spot)
    # call_payoff1 = call.payoff()
    # print('payoff:', call_payoff1)
    
    # call.set_spot(np.array(spot))
    # call_payoff2 = call.payoff()
    # print('payoff:', call_payoff2)
    
    # call.set_spot(spots)
    # call_payoff3 = call.payoff()
    # print('payoffs:', call_payoff3)
    
    # call.set_spot(spot)
    # call_profit1 = call.profit()
    # print('profit:', call_profit1)
    
    # call.set_spot(np.array(spot))
    # call_profit2 = call.profit()
    # print('profits:', call_profit2)
    
    # call.set_spot(spots)
    # call_profit3 = call.profit()
    # print('profits:', call_profit3)

    # call.set_market_inputs(spot, vol, r)
    # call_pricing1 = call.pricing_bs()
    # print('pricing:', call_pricing1)
    
    # call.set_market_inputs(np.array(spot), vol, r)
    # call_pricing2 = call.pricing_bs()
    # print('pricings:', call_pricing2)
    
    # call.set_market_inputs(spots, vol, r)
    # call_pricing3 = call.pricing_bs()
    # print('pricings:', call_pricing3)
    
    
    
    # import matplotlib.pyplot as plt
    
    # fig, ax = plt.subplots()
    # ax.plot(spots, call_payoff3)
    # ax.plot(spots, call_pricing3)
    # plt.show()
    
    # fig, ax = plt.subplots()
    # ax.plot(spots, put.pricing_bs(spot=spots, vol=vol, r=r, dt=dt))
    # ax.plot(spots, put.payoff(spot=spots))
    # plt.show()
    
    
    # s = np.array([96, 98, 100, 102, 104])
    # print('pricing:', call.payoff(spot=s))


